<?php
    
namespace App\Http\Controllers\Admin;

use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\StoreUsersRequest;
use App\Http\Requests\Admin\UpdateUsersRequest;

use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use DB;
use Hash;
use DataTables;

use App\Imports\UsersImport;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Arr;

use Brian2694\Toastr\Facades\Toastr;
    
class RolesController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    
    function __construct()
    {
         //$this->middleware('permission:role-list|role-create|role-edit|role-delete', ['only' => ['index','store']]);

   
    }
    

    public function index(Request $request)
    {
        $roles = Role::all();

        return view('admin.roles.index', compact('roles'));
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $allpermissions = Permission::orderBy('id', 'ASC')->get();
        return view('admin.roles.create', compact('allpermissions'));
    }
    
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->validate($request, [
            'name' => 'required|unique:roles,name',
            'permission' => 'required',
        ]);
    
        $role = Role::create(['name' => $request->input('name')]);
        $role->syncPermissions($request->input('permission'));

        Toastr::success('Role Added Successfully :)','User Management');

        

        return redirect()->route('admin.roles.index')
                        ->with('success','Role created successfully');
    }
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $role = Role::find($id);
        $permission = Permission::get();
        $permissions = Permission::get()->pluck('name', 'name');

        $rolePermissions = DB::table("role_has_permissions")->where("role_has_permissions.role_id",$id)
            ->pluck('role_has_permissions.permission_id','role_has_permissions.permission_id')
            ->all();

        $allpermissions = Permission::orderBy('id', 'ASC')->get();

        return view('admin.roles.show', compact('role', 'permission', 'permissions',  'rolePermissions', 'allpermissions'));
    }
    
    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $role = Role::find($id);
        $permission = Permission::get();
        $permissions = Permission::get()->pluck('name', 'name');

        $rolePermissions = DB::table("role_has_permissions")->where("role_has_permissions.role_id",$id)
            ->pluck('role_has_permissions.permission_id','role_has_permissions.permission_id')
            ->all();

        $allpermissions = Permission::orderBy('id', 'ASC')->get();

        return view('admin.roles.edit', compact('role', 'permission', 'permissions',  'rolePermissions', 'allpermissions'));
    }
    
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $this->validate($request, [
            'name' => 'required',
            'permission' => 'required',
        ]);
    
        $role = Role::find($id);
        $role->name = $request->input('name');
        $role->save();
    
        $role->syncPermissions($request->input('permission'));
    
        Toastr::success('Role Updated Successfully :)','User Management');
        return redirect()->route('admin.roles.index')
                        ->with('success','Role updated successfully');
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        DB::table("roles")->where('id',$id)->delete();

        Toastr::warning('Role Deleted Successfully :)','User Management');
        return redirect()->route('admin.roles.index')
                        ->with('success','Role deleted successfully');
    }

    public function allRoles(Request $request)
    {
        
        $columns = array( 
                            0 =>'id', 
                            1 =>'name',
                            4=> 'id',
                        );
  
        $totalData = Role::count();
            
        $totalFiltered = $totalData; 

        $limit = $request->input('length');
        $start = $request->input('start');
        $order = $columns[$request->input('order.0.column')];
        $dir = $request->input('order.0.dir');
            
        if(empty($request->input('search.value')))
        {            
            $posts = Role::offset($start)
                         ->limit($limit)
                         ->orderBy($order,$dir)
                         ->get();
        }
        else {
            $search = $request->input('search.value'); 

            $posts =  Role::where('id','LIKE',"%{$search}%")
                            ->orWhere('name', 'LIKE',"%{$search}%")
                            ->offset($start)
                            ->limit($limit)
                            ->orderBy($order,$dir)
                            ->get();

            $totalFiltered = Role::where('id','LIKE',"%{$search}%")
                             ->orWhere('name', 'LIKE',"%{$search}%")
                             ->count();
        }

        $data = array();
        if(!empty($posts))
        {
            foreach ($posts as $post)
            {
                $show =  route('admin.roles.show',$post->id);
                $edit =  route('admin.roles.edit',$post->id);
                $delete =  route('admin.roles.destroy',$post->id);
                $token =  csrf_token();
                /*    
                  $row[5] = '  <div class="btn-group">
                    <button aria-expanded="false" aria-haspopup="true" data-toggle="dropdown" class="btn-xs btn-info dropdown-toggle" type="button">
                    Action 
                    <span class="caret"></span></button>

                    <div class="dropdown-menu">
                      <a class="dropdown-item" href="{{ route(admin.roles.show,$post->empno) }}">View</a>
                      <a class="dropdown-item" href="">Edit</a>
                      <a class="dropdown-item" href="" onclick="return confirm(\'Are you sure to Delete Selected Item?\'); ">Delete</a>
                    </div>      
                  </div>'; 
                */    

                  $permissions = "";
                  foreach($post->permissions()->pluck('name') as $permission){
                        $permissions = $permissions. '<span class="badge badge-info">'.$permission. '</span>';   
                   }; 

                   $row[6] = $permissions;
                  
                  //$row[6] = foreach($post->roles()->pluck('name') as $role){
                   //             '<span class="badge badge-info">{{ $role }}</span>'
                   //                             }                           


                $nestedData['id'] = $post->id;
                $nestedData['name'] = $post->name;
                $nestedData['permission'] = $row[6];

                //@endforeach
                //$nestedData['role'] = $post->roles()->pluck('name');
  
                //$nestedData['options'] = "&emsp;<a href='{$show}' title='SHOW' >SHOW<span class='btn btn-primary btn-mini waves-effect waves-light'></span></a>
                //                        &emsp;<a href='{$edit}' title='EDIT' >EDIT<span class='fa fa-user'></span></a>";
                $nestedData['options'] = "<a href='{$show}' title='View' class='m-r-15 crm-action-edit text-muted'><i class='fas fa-desktop'></i></a>  &emsp;<a href='{$edit}' title='Edit' class='m-r-15 crm-action-edit text-muted'><i class='fas fa-edit'></i></a>  &emsp;". 
                    "<form action='{$delete}' method='POST' onsubmit='return confirm(\'Are you Sure?\')'; style='display: inline-block;'>".
                        "<input type='hidden' name='_method' value='DELETE'>".
                        "<input type='hidden' name='_token' value='{$token}'>". 
                        "<button type='submit' title = 'Delete' class='crm-action-delete text-muted' style='border:none; background:none; cursor:pointer;'><i class='fas fa-trash-alt'></i></button>".
                    "</form>";
                    //<input type="button" onclick="location.href='https://google.com';" value="Go to Google" />
                    //<a href='{$edit}' title='Delete' class='crm-action-delete text-muted'><i class='icofont icofont-delete-alt'></i></a>";
                    //<button type='submit' class='crm-action-delete text-muted'><i class='icofont icofont-delete-alt'></i></button>
                                                        
                //$nestedData['options'] = "<center>". $row[5]. "</center>";
                $data[] = $nestedData;

            }
        }
          
        $json_data = array(
                    "draw"            => intval($request->input('draw')),  
                    "recordsTotal"    => intval($totalData),  
                    "recordsFiltered" => intval($totalFiltered), 
                    "data"            => $data   
                    );
            
        echo json_encode($json_data); 
        
    }  
}