<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Providers\RouteServiceProvider;
use App\Models\User;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    //protected $redirectTo = RouteServiceProvider::HOME;
    protected $redirectTo = 'customer/dashboard';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'email' => ['required', 'string', 'email', 'max:255', 'unique:users'],
            'password' => ['required', 'string', 'min:8', 'confirmed'],
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\Models\User
     */
    protected function create(array $data)
    {
        if($data['suffix'] == ""){
            $suffix = "";
        } else {
            $suffix = $data['suffix'];
        }
        $name = $data['fname']. " ". substr($data['mname'], 0, 1). " ". $data['lname']. " ". $suffix;
        return User::create([
            'name' => $name,
            'lname' => $data['lname'],
            'fname' => $data['fname'],
            'mname' => $data['mname'],
            'suffix' => $data['suffix'],
            'email' => $data['email'],
            'password' => Hash::make($data['password']),
            'mobileno' => $data['mobileno'],
            'phoneno' => $data['phoneno'],
            'birthdate' => $data['birthdate'],
            'birthplace' => $data['birthplace'],
            'gender' => $data['gender'],
        ]);
    }
}
